import pandas as pd
from typing import Generator
from .is_last import is_last
from .get_jumps import get_jumps


def text_positions(df: pd.DataFrame, bar_width: float, index_level: int) -> Generator:
    labels_offsets = {}
    old_index = tuple()
    bar_position = 0
    for i, (index, _) in enumerate(df.iterrows()):
        if isinstance(index, str):
            index = (index,)
        try:
            iter(index)
        except TypeError:
            index = (index,)
        jumps = get_jumps(df, i, index, old_index)
        if not old_index:
            old_index = index
        old_bar_position = bar_position
        for j, value in enumerate(jumps):
            if value:
                text_position = old_bar_position
                if j in labels_offsets:
                    text_position += labels_offsets[j]
                if is_last(df, i):
                    if j != len(jumps)-1:
                        text_position += bar_width
                else:
                    bar_position += bar_width
                if j == index_level:
                    yield text_position/2 + bar_width/2, old_index[j]
        if j == index_level and j == len(jumps)-1 and is_last(df, i):
            yield text_position/2 + bar_width*1.5, index[j]
        old_index = index
        for j, value in enumerate(jumps):
            if value:
                labels_offsets[j] = bar_position



import pandas as pd
from typing import Union, List, Any
from .is_last import is_last


def get_jumps(df: pd.DataFrame, row: int, index: Union[List, Any], old_index: Union[List, Any]) -> List[bool]:
    """Return list representing the detected jumps from given index and old_index.

    Parameters
    ----------
    df: pd.DataFrame,
        Dataframe from which to detect index jumps.
    row: int,
        Current row number.
    index:Union[List, Any],
        List of indices of index curresponding to given row number.
    old_index:Union[List, Any],
        Old index.

    Returns
    -------
    Returns list of boolean representing if for given index level a jump has been detected.
    """
    if not old_index:
        return [False]*len(index)

    previous = False
    jumps = []
    for new, old in zip(index, old_index):
        jumps.append(
            new != old or previous
        )
        previous = new != old
    return jumps



import pandas as pd


def is_last(df: pd.DataFrame, row: int) -> bool:
    """Return boolean representing if given row is the last row.

    Parameters
    ----------
    df: pd.DataFrame,
        The dataframe to check against.
    row: int,
        The index of the row to consider.

    Returns
    -------
    A boolean representing if given row is the last one.
    """
    return row+1 == df.shape[0]
